<?php
// +----------------------------------------------------------------------
// | 启动定时器
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://xiuxian.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------
ini_set("display_errors", "On");//打开错误提示
ini_set("error_reporting",E_ALL);//显示所有错误

$obj = new task();
$obj->run();

class task {

    /**
     * 任务菜单
    */
    private $_task = [];
    /**
     * 项目跟地址
    */
    private $_path;
    /**
     * 秒
    */
    private $second;
    /**
     * 分钟
    */
    private $minute;
    /**
     * 小时
    */
    private $hour;
    /**
     * 天数
    */
    private $day;
    /**
     * 月份
    */
    private $month;
    /**
     * 星期
    */
    private $week; 
    /**
     * 任务进程池，这块可以按你的任务数来扩展启动，防止无限创建进程，demo暂时统一使用一个
    */
    private $process = null;
   
    /**
     * 启动入口
    */
    public function run() {
        $this->_path = dirname(dirname(__FILE__));
        // 开跑
        $this->get_task();
        $this->go_task();
    }

    /**
     * 获取任务列表
    */
    private function get_task() {
        # 这里应该加入redis，按分钟缓存从数据库查询出来的任务列表，否注会大量并发查询数据库
        $this->_task = require_once $this->_path.'/sql.php';
    }

    /**
     * 启动一个秒级的定时器，用于每1000毫秒，读取一次任务脚步
    */
    private function go_task() {
        swoole_timer_tick(1000, function ($timer_id) {
            # 获取系统时间
            $this->get_data_list();
            # 循环任务列表
            foreach ($this->_task as $v) {
                # 验证并执行任务脚步
                $this->task_vif($v);
            }
        });
    }
    
    /**
     * 验证并执行任务脚步
     * @param array $v 任务记录
     * @param mixed 无
    */
    private function task_vif($v) {
        // 秒级特殊规则
        if ($v['second']!='*' && $v['month']=='*' && $v['week']=='*' && $v['day']=='*' && $v['hour']=='*' && $v['minute']=='*') {
            # 每*秒运行一次
            if (is_int($this->second / $v['second']) === false) {
                return false;
            }
        // 分级特殊规则
        } else if ($v['minute']!='*' && $v['month']=='*' && $v['week']=='*' && $v['day']=='*' && $v['hour']=='*') {
            # 每*分运行一次
            if ($v['second'] != '*') {
                if ($this->second != $v['second'] || is_int($this->minute / $v['minute']) === false) {
                    return false;
                }
            } else if (is_int($this->minute / $v['minute']) === false) {
                return false;
            }
        // 普通规则开始计算
        } else {
            // 参数预设
            if ($v['month'] != '*' && $v['day'] == '*') $v['day'] = 1; // 第1天
            if ($v['hour'] == '*') $v['hour'] = 0; // 0时
            if ($v['minute'] == '*') $v['minute'] = 0; // 0分
            if ($v['second'] == '*') $v['second'] = 1; // 1秒

            # 计算月份
            if ($v['month'] != '*' && $this->month != $v['month']) {return false;}
            # 计算星期
            if ($v['week'] != '*' && $this->week != $v['week']) {return false;}
            # 计算天数
            if ($v['day'] != '*' && $this->day != $v['day']) {return false;}
            # 计算小时
            if ($v['hour'] != '*' && $this->hour != $v['hour']) {return false;}
            # 计算分钟，用除法
            if ($v['minute'] != '*' && $this->minute != $v['minute']) {return false;}
            # 计算秒，用除法
            if ($v['second'] != '*' && $this->second != $v['second']) { return false;}
        }

        # 上面规则都通过，则启动一个10毫秒后启动的一次性定时器
        swoole_timer_after(10, function () use ($v) {
            # 解析任务脚步
            $pid = swoole_async::exec($v['task'], function ($result, $status) use ($v) {
                # 是否需要记录日志文件
                # 这里可以根据任务ID，自动写入文件
                if ($v['bin_log']) {
                    $path = $this->_path.'/bin_log/'.date('Y-m-d').'/';
                    if (!file_exists($path)) {
                        mkdir($path, '0755');
                    }
                    $file = $path.$v['id'].'.log';
                    error_log($result.date('Y-m-d H:i:s')."\r\n", 3, $file);
                }
            });
        });
    }

    /**
     * 获取当前系统时间，分割
    */
    private function get_data_list() {
        $time = time();
        # 秒
        $this->second = $this->delete_zero(date('s', $time));
        # 分钟
        $this->minute = $this->delete_zero(date('i', $time));
        # 小时
        $this->hour = $this->delete_zero(date('H', $time));
        # 天数
        $this->day = $this->delete_zero(date('d', $time));
        # 月份
        $this->month = $this->delete_zero(date('m', $time));
        # 星期 (0就是周末)
        $this->week = date('w', $time);
    }

    /**
     * 删除开头0
    */
    private function delete_zero($str) {
        return intval($str);
    }

}
